/* lintian.css -- Style sheet for lintian.debian.org pages. */

/*
 # Table of Contents:
 #
 # 1. General styles (links, lists, titles, tables...)
 # 2. Header
 # 3. Navigation
 # 4. Main content
 # 5. Footer
 # 6. Other
 */

/*
 # Order:
 #
 # example {
 #   display
 #   position
 #   width
 #   height
 #   margin
 #   padding
 #   background
 #   color
 #   font
 #   text
 #   line-height
 #   border
 # }
 */


/*
 * 1. General styles
 */

* {
  -webkit-box-sizing: border-box;
     -moz-box-sizing: border-box;
          box-sizing: border-box;
}

html, body {
    margin: 0;
    padding: 0;

    font-family: 'Roboto', serif;
    color: #222;
}

h1, h2 {
    font-family: 'Roboto Condensed', sans-serif;
    font-weight: 400;
}

h1 {
    margin-top: 0;
}

article h1 {
    font-size: 28px;
}

h2 {
    margin-top: 25px;
    margin-bottom: 14px;
    padding-bottom: 6px;
    border-bottom: 2px solid #AAA;
}

h3 {
    margin: 5px 0 5px 10px;
    color: #444;
    font-size: 1.0em;
}

a {
    color: #3252B2;
}

ul {
    margin: 0;
}

li {
    color: #333;
}

hr {
    display: none;
}

/*
 * 2. Header title
 */

body > header {
    background: #F3F3F3 url("/images/logo-small.png") no-repeat right;
    border-bottom: 1px solid #AAA;

    display: flex;
    justify-content: space-between;
    align-items: center;

    color: #333;

    height: 50px;
    padding-right: 85px; /* space for logo */
}

body > header h1 {
    font-family: 'Roboto', serif;
    font-size: 1.2em;
    margin: 20px 20px;
}

body > header a {
    color: inherit;
    text-decoration: none;
}

body > header nav ul {
    list-style: none;
    margin: 0;
    padding: 0;
}

body > header nav li {
    display: inline-block;
}

body > header nav a {
    display: block;
    padding: 3px 5px;
    font-size: 0.9em;
    border-bottom: 2px solid #f3f3f4;
}

body > header nav a:hover {
    border-bottom: 2px solid #D70751;
}

/*
 * 4. Main content
 */

main {
    margin: 40px 40px 0;
    font-size: 0.9em;
}

.showcase {
    margin: 25px 0;
    padding: 40px 20px;
    background: #EFF4F8 url("/images/l.png") no-repeat left;
    background-origin: content-box;
    border: 1px solid #DFE4E8;
}

.showcase p {
    margin-left: 160px;
    font-size: 1.1em;
}

/* For smaller devices, we leave out the "icon" - it is fairly wide with
 * little added value.
 */
@media (max-width: 700px) {

  .showcase {
      margin: 25px 0;
      padding: 20px;
      background: #EFF4F8;
      border: 1px solid #DFE4E8;
  }

  .showcase p {
      margin-left: 5%;
  }
}

li span {
    margin: 0 0px 0 0;
    padding: 1px 0px;
}

/* E/W/I colors */

span.label {
    font-family: 'Roboto Condensed', sans-serif;
    font-weight: 700;
}

article#hints {
    margin-top: 20px;
    padding-top: 10px;
    border-top: 2px solid #AAA;
    clear: both;
}

.hint.tiny {
    font-family: 'Roboto Condensed', sans-serif;
    font-weight: 700;
    font-size: 0.9em;
}

.context.tiny {
    font-family: 'Roboto', sans-serif;
    font-style: italic;
    font-size: 0.9em;
}

.comment.tiny {
    font-family: 'Roboto', sans-serif;
    font-size: 0.9em;
}

ul.minor {
  list-style-type: none;
}

ul.see-also {
    padding-bottom: 1em;
}

ul.taglist {
  list-style-type: none;
}

ul.screenlist {
  list-style-type: none;
}

ul.sourcelist {
  list-style-type: none;
}

ul.filelist {
    list-style-type: disc;
    margin-left: 2em;
}

ul.hintlist {
    list-style: none;
    padding-left: 10px;
}

ul.commentlist {
    list-style: none;
    padding-left: 0px;
}

li.fileitem {
    margin: 10px 0 5px 0;
}

span.visibility-symbol {
    font-family: monospace;
    font-size: 1.1em;
}

a.visibility-symbol {
    text-decoration: none;
}

a.visibility-symbol:hover {
    filter: invert(100%);
}

a.visibility-symbol.big {
    padding: 1px 5px;
    margin-right: 5px;
    font-weight: normal;
}

a.visibility-symbol.tiny {
    font-weight: bold;
    padding: 1px 3px;
}

a.visibility-symbol::before {
    border: 1px solid;
    display: inline-block;
    width: 1em;
    font-family: monospace;
    font-size: 1.1em;
    text-align: center;
}

a.visibility-symbol.error::before {
    content: 'E';
    color: #fff;
    border-color: #fff;
    background-color: #fc0905;
}

a.visibility-symbol.warning::before {
    content: 'W';
    color: #111;
    border-color: #888;
    background-color: #fceb02;
}

a.visibility-symbol.info::before {
    content: 'I';
    color: #fff;
    border-color: #fff;
    background-color: #465efc;
}

a.visibility-symbol.pedantic::before {
    content: 'P';
    color: #fff;
    border-color: #fff;
    background-color: #00c627;
}

a.visibility-symbol.experimental::before {
    content: 'X';
    color: #fff;
    border-color: #fff;
    background-color: #9e8040;
}

a.visibility-symbol.mask::before {
    content: 'M';
    color: #111;
    border-color: #444;
    background-color: #ddd;
}

a.visibility-symbol.override::before {
    content: 'O';
    color: #fff;
    border-color: #fff;
    background-color: #444;
}

a.visibility-symbol.classification::before {
    content: 'C';
    color: #fff;
    border-color: #fff;
    background-color: #bc55fc;
}

section.explanation {
    padding: 8px 28px;
    background-color: #EEE;
    border: 1px solid #BBB;
    float: left;
    clear:left;
}

.explanation.error {
    background-color: #fccac9;
    border: 1px solid #fc0905;
}

.explanation.warning {
    background-color: #fcf9c9;
    border: 1px solid #fceb02;
}

.explanation.info {
    background-color: #dee2fc;
    border: 1px solid #465efc;
}

.explanation.pedantic {
    background-color: #d4fcdc;
    border: 1px solid #00c627;
}

.explanation.experimental {
    background-color: #fcf2de;
    border: 1px solid #9e8040;
}

.explanation.classification {
    background-color: #f4e8fc;
    border: 1px solid #bc55fc;
}

section.auxiliary {
    margin: 20px 0;
    float: left;
}

div.terminal {
    margin-bottom: 1em;
}

p.terminal {
    font-family: monospace;
    margin-top: 0;
    margin-bottom: 0;
}

article.version {
    display: flex;
    gap: 1em;
    align-items: flex-start;
}

article.version nav.version-selector {
    flex: 0 auto;
    order: 2;
}

nav.version-selector {
    background-color: #F3F3F3;
    border: 1px solid #AAA;
    padding: 0.8em;
    margin: 0 0 20px 20px;
    display: inline-block;
    float: right;
}

.version-selector h3 {
    font-weight: bold;
    text-align: center;
    white-space: nowrap;
    margin: 0;
    margin-bottom: 0.5em;
}

.version-selector ul {
    list-style: none;
    padding: 0;
}

.version-selector li {
    white-space: nowrap;
}

.version-selector span.highlight {
    color: #D70751;
}

span.maintenance_links {
    font-size: 0.6em;
    margin-left: 1.2em;
}

.liberty.main {
}

.liberty.contrib {
    color: red;
}

.liberty.non-free {
    color: red;
}

span.release-list {
    font-style: italic;
}

span.quote {
    font-style: italic;
}

span.run-details {
    font-style: italic;
}

a.sample {
    font-style: italic;
}

/*
 * 5. Footer
 */

footer {
    display: block; /* HTML5 compat */
    margin: 20px 20px;
    padding: 10px 0 0 0;
    font-family: 'Roboto Condensed', sans-serif;
    font-size: 0.85em;
    border-top: 1px solid #AAA;
}

footer p {
    margin: 0;
    padding: 0;
}
